
# Complex.pm
# (C) 2006, Alex S.

package Complex;
use strict;

use overload (
    'fallback'  => 1,
    '""'  => 'as_string',
    '-'   => 'subtract',
    '+'   => 'add',
    '*'   => 'mult',
    '/'   => 'div',
); 

sub new {
    my ($class, %args) = @_;
    return bless { %args }, $class;
}

sub as_string {
    my ($this) = @_;
    return "(".$this->{r}."+".$this->{i}."i)";
}

sub add {
    my ($a, $b) = @_;        
    unless (UNIVERSAL::isa($b, 'Complex')) {
        die "Can only add two Complex objects, not $b";
    }
    return new Complex(r=>($a->{r}+$b->{r}),i=>($a->{i}+$b->{i}));
}

sub subtract {
    my ($a, $b) = @_;
    unless (UNIVERSAL::isa($b, 'Complex')) {
        die "Can only subtract two Complex objects, not $b";
    }
    return new Complex(r=>($a->{r}-$b->{r}),i=>($a->{i}-$b->{i}));
}

sub mult {
    my ($a, $b) = @_;
    unless (UNIVERSAL::isa($b, 'Complex')) {
        die "Can only multiply two Complex objects, not $b";
    }
    return new Complex(
        r=>($a->{r} * $b->{r} - $a->{i} * $b->{i}),
        i=>($a->{r} * $b->{i} + $a->{i} * $b->{r})
    );
}

sub div {
    my ($a, $b) = @_;
    unless (UNIVERSAL::isa($b, 'Complex')) {
        die "Can only multiply two Complex objects, not $b";
    }
    return new Complex(
        r=>( ($a->{r} * $b->{r} + $a->{i} * $b->{i})/
            ($b->{r}*$b->{r} + $b->{i}*$b->{i}) ),
        i=>( ($a->{i} * $b->{r} - $a->{r} * $b->{i})/
            ($b->{r}*$b->{r} + $b->{i}*$b->{i}) )
    );
}

1;

