#!/usr/bin/perl

# ftmult.pl
# (C) 2006, Alex S

use Complex;
use strict;

# the stupid O(n^2) version of the fourier transform.
# s represents the sign (-1 is forward, +1 is backwards)
# returns the transformed array.
sub ft {
    my ($f,$s) = @_;
    my $F = [];
    for my $i (0..$#$f){
        $F->[$i] = new Complex();
        for my $j (0..$#$f){
            my $t = $j * $i * $s * 2.0 * 3.14159265 / ($#$f + 1);
            # print "theta: $t\n";
            $F->[$i]=$F->[$i]+($f->[$j]*(new Complex(r=>cos($t),i=>sin($t))));
        }
    }
    map { $_->{r} /= ($#$f+1); $_->{i} /= ($#$f+1); } @$F if $s > 0;
    return $F;
}

# the multiplication that uses the slow/stupid fourier transform.
sub mlt {
    my ($a,$b) = @_;
    my $m = (length($a) > length($b) ? length($a) : length($b)) * 2;
    $a = ('0'x($m-length $a)).$a; $b = ('0'x($m-length $b)).$b;
    my $a_f = ft([map { new Complex(r=>$_) } reverse split //,$a],-1);
    my $b_f = ft([map { new Complex(r=>$_) } reverse split //,$b],-1);
    map { $a_f->[$_] = $a_f->[$_] * $b_f->[$_] } 0..$#$a_f;
    my $cr = 0;     # carry
    return join ('', reverse map { 
            my $a=int($_->{r} + 0.5)+$cr; 
            $cr=int($a/10); 
            $a % 10 
        } @{ft($a_f,1)}
    );
}

# print the results...
print mlt(
    '389432903702309750817934209340574740496093487',
    '239567296501328978409756749087430897609745092')."\n";

